function x = cg_step(A, rhs, x, s, maxit=0, f=[])


[m,n] = size(A);
r = rhs - A * x;
ean = -1;
[e, ean] = update_e(A, f, x, ean);


% init
R(:,1) = r;
for i=2:s
	R(:,i) = A * R(:,i-1);
end
W = c_W(R, A); % compute W
res = norm(r);
P = zeros(m,s);
B = zeros(s,s);



i=1;
while i<maxit && res>1e-6
	muV = c_muV(s, r, A); 
	mV = muV(1:s,:);

	if i!= 1
		%C = - c_cVnew(r, a, muV);
		C = - c_cV(r, A, P);

		B(:,1) = W \ C(:,1);
		for j=2:s
			B(:,j) = W \ C(:,j);
		end

		W = c_Wnew(muV, B, -C);
	end

	a = W \ mV;

	%Pprev = P;
	P = R + P * B;
	% verification
	%for j=1:s
	%	p = transpose( P(:,j) );
	%	for jj=1:s
	%		an = p * A * Pprev(:,jj);
	%		if an > 1e-8
	%			printf("warning: new p%i not A-adjoint to p%i (%16.20e)\n", j, jj, an);
	%		end
	%	end	
	%end
	%
	 

	x += P * a
	r = rhs - A * x;

	% verification
	%for j=1:s
	%	if transpose(r) * P(:,j) > 1e-10
	%		printf("warning: r not orthogonal with p%i\n",j);
	%	end
	%end
	% 


	R(:,1) = r;
	for j=2:s
		R(:,j) = A * R(:,j-1);
	end


	[e, ean] = update_e(A, f, x, ean);
	res = norm(r);

	printf("%2i: %16.20e    %f\n", i, res, ean);

	i += 1;
end

endfunction



% Compute W, the sxs matrix of inner products (p^l, AP^j)
function W = c_W(P, A)

[~,s] = size(P);
W = zeros(s,s);

for i=1:s
	for j=1:s
		W(i,j) = transpose(P(:,i)) * A * P(:,j);
	end
end

endfunction


function W = c_Wnew(muV, B, C)

[~,s] = size(B);

for i=1:s
	for j=1:s
		W(i,j) = muV(i+j,1) + transpose(B(:,i)) * C(:,j);
	end
end

endfunction
% 



% Computation of c1, c2, ... cs
function C = c_cV(r, A, P)

[~,s] = size(P);

for j=1:s
	D = A^j;
	for i=1:s
		C(i,j) = transpose(r) * D * P(:,i);
	end
end

endfunction


function c = c_cVnew(r, a, muV)

[s,~] = size(a);
c = zeros(s,s);
c(s,1) = - transpose(r) * r / a(s,1);
for j=2:s
	prev = c(:,j-1);
	c(1:s-1,j) = prev(2:s);

	tmp = muV(j,1);
	for i=1:j-1
		tmp += c(s-j+i,j) * a(s-j+i,1);
	end
	c(s,j) =  - tmp  / a(s,1);
end

endfunction
% 




function [e,nean] = update_e(A, f, x, ean)
if size(f) != [0,0]
	e = f - x;
	nean = norm( transpose(e) * A * e );

	if nean > ean && ean!=-1
		printf("A-norm of err increases!\n");
	end
else
	e = -1;
	nean = -1;
end
endfunction




% Computation of the moments of r: { (r,r) , (r,Ar) , (r,A^2 r) , ... }
function m = c_muV(s, r, A)

m(1,1) = transpose(r) * r;

tmp = r;
for i=2:2*s+1
	tmp = A * tmp;
	m(i,1) = transpose(r) * tmp; 
end

endfunction




function mV = c_mV(r, P)

[~,s] = size(P);

for i=1:s
	mV(i,1) = transpose(r) * P(:,i);
end

endfunction
