## usage: x = cgmod1 (A, b, x, maxit=0, rtol=1e-16, f=[])


function x = cgmod1(A, b, x, maxit=0, rtol=1e-16, f=[])


[m,n] = size(A);

if maxit==0
	maxit=n;
end

r = b - A * x;
p = r;
gamma = transpose(r) * r;
v = A * p;
sigma = transpose(p) * v;
x = x + gamma / sigma * p;
r = r - gamma / sigma * v;

i=0;
tol = sqrt( gamma );

while i<maxit && tol>rtol
	s = A * r;

	prevgamma = gamma;
	gamma = transpose(r) * r;
	delta = transpose(r) * s;

	beta = gamma / prevgamma;
	p = r +	beta * p;
	v = s + beta * v;
	sigma = delta - beta * beta * sigma; 
	alpha = gamma / sigma;

	x += alpha * p;
	r -= alpha * v;

	tol = sqrt(gamma);
	printf("%i %16.20e %16.20e %16.20e %16.20e\n", i, tol, sigma, alpha, delta);
	
	prevgamma = gamma;
	i = i+1;
end
