#include "task_gemm.h"

#include "fptype.h"
#include "fpblas.h"
#include "blas.h"
#include "async_struct.h"
#include "selfsched.h"
#include "prof.h"
#include "task_log.h"


#ifdef DOUBLE_PRECISION

#define __t_gemm			task_dgemm
#define __t_gemmcg			task_dgemmcg
#define __t_gemm_prof		task_dgemm_prof
#define __t_gemm_sched		task_dgemm_sched

#else

#define __t_gemm			task_sgemm
#define __t_gemmcg			task_sgemmcg
#define __t_gemm_prof		task_sgemm_prof
#define __t_gemm_sched		task_sgemm_sched

#endif


// C = alpha * A * B - beta * C
// A bm x bk 
// B bk x bn 
// C bm x bn 
void __t_gemmcg(int it, async_t *sync, ompssblas_t transa, ompssblas_t transb, int bm, int bn, int bk, fp_t alpha, fp_t *A, int lda, fp_t *B, int ldb, fp_t beta, fp_t *C, int ldc, int p, int idx) 
{
	if ( idx != -1 ) {
		pthread_mutex_t *mutex = &sync->mutex;
		pthread_mutex_lock(mutex);

		log_record(sync, it, EVENT_PRIORITY, idx, (float)p);

		pthread_mutex_unlock(mutex);
	}
	BLAS_gemm(transa, transb, bm, bn, bk, alpha, A, lda, B, ldb, beta, C, ldc);
}

void __t_gemm(ompssblas_t transa, ompssblas_t transb, int bm, int bn, int bk, fp_t alpha, fp_t *A, int lda, fp_t *B, int ldb, fp_t beta, fp_t *C, int ldc, int p) 
{
	BLAS_gemm(transa, transb, bm, bn, bk, alpha, A, lda, B, ldb, beta, C, ldc);
}

#if 0
void __t_gemm_sched(int bm, int bn, int bk, fp_t alpha, fp_t *A, int lda, int i, selfsched_t *schedB, fp_t *B, int ldb, fp_t beta, fp_t *C, int ldc, int p) 
{
	//sched_wait(schedB, i);

	BLAS_gemm(OMPSSBLAS_NTRANSP, OMPSSBLAS_NTRANSP, bm, bn, bk, alpha, A, lda, B, ldb, beta, C, ldc);
}


void __t_gemm_prof(int it, async_t *sync, int idx, int p, int bm, int bn, int bk, int m, int n, int k, fp_t alpha, fp_t *A, int lda, 
	selfsched_t *schedB, fp_t *B, int ldb, fp_t beta, fp_t *C, int ldc) 
{
	int profs = sync->prof.s;
	int childp = 1;

	if ( profs ) {
		pthread_mutex_t *mutex = &sync->mutex;
		pthread_mutex_lock(mutex);

		/* get profile ... */
		childp = prof_priority(&sync->prof, idx);
		/* ... log it */
		log_record(sync, it, EVENT_PRIORITY, idx, childp);
		printf("haha!\n");

		pthread_mutex_unlock(mutex);
	}

	__t_gemm_sched(bm, bn, bk, alpha, &A[0*m], lda, 0, schedB, B, ldb, beta, C, ldc, childp);

	int i;
	int l;
	for ( l=bk, i=1; l<k; l+=bk, ++i ) {
		int fs = k - l;
		int f = fs < bk ? fs : bk;

		__t_gemm_sched(bm, bn, f, alpha, &A[l*m], lda, i, schedB, &B[l], ldb, FP_ONE, C, ldc, childp);
	}
#pragma omp taskwait
}
#endif
