#ifndef __BSBLAS_CSRMM_H__
#define __BSBLAS_CSRMM_H__


#include "fptype.h"
#include "hb.h"
#include "task_csrmm.h"
#include "matfprint.h"
#include "selfsched.h"


#ifdef SINGLE_PRECISION

#define BSBLAS_CSRMM				bsblas_scsrmm

#else

#define BSBLAS_CSRMM				bsblas_dcsrmm

#endif


/* A hb csr, B and C dense col major */
static inline void __attribute__((always_inline)) bsblas_scsrmm(int b, int n, float alpha, hbmat_t *Ahbh, float *B, int ldb, float beta, float *C, int ldc, selfsched_t *sched) 
{
	int M = Ahbh->m;
	int N = Ahbh->n;

	int *vptr = Ahbh->vptr;
	int *vpos = Ahbh->vpos;
	hbmat_t **vval = Ahbh->vval;
	int offs = vptr[0] == 0 ? 0 : 1;

	int cmaj = 1;

	int I;
	for ( I = 0; I < M; ++I ) {
		int is = vptr[I] - offs;  
		int bcnt = vptr[I+1] - vptr[I];
		int ib = vval[is]->m;
		
		int J;
		for ( J = 0; J < n; J+=b ) {
			int boffs = cmaj ? J * ldb : 0 ;
			int coffs = cmaj ? J * ldb + I*b : 0 ;
			int jleft = n - J;
			int c = jleft < b ? jleft : b;

			float *bC = &(C[coffs]);
			float *bB = &(B[boffs]);

			/* ib = number of rows in block row I */
			/* c = number of cols in block  col J */
			task_scsrmm_sched(bcnt, ib, c, b, alpha, &vptr[I], &vpos[is], &vval[is], bB, ldb, beta, bC, ldc, sched);
		}
	}
}


#endif // __BSBLAS_CSRMM_H__
