#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <sys/time.h>

#include "itref_config.h"
#include "itref_setup.h"
#include "fptype.h"
#include "fpblas.h"
#include "task_log.h"
#include "as_man.h"
#include "ompss_apps.h"
#include "ompss_cg.h"

#include "bblas_copy.h"
#include "bblas_gemm.h"
#include "matfprint.h"
#include "bblas_axpy.h"
#include "bblas_convert.h"
#include "bblas_dot.h"
#include "bblas_util.h"


int n; /* dimension of A */
int bm; /* row block size */
int bn; /* column block size */
int s; /* number of right-hand sides or columns in b */
int refit; /* maximum number of iteration steps */
int it; /* maximum iteration count */
double prec; /* required precision */
int rep; /* number of times to repeat the CG */

double *A; /* coefficient matrix */
double *rhs; /* right-hand side */ 
double *x[2]; /* solution */
//double *xstar; /* exact solution */
float *work; /* work array */
char *rhsfname;
char *aname;
unsigned long works;

int lookahead;
int async;
double profile;
int warmup;
int release;
double orth_fac;


int main (int argc, char *argv[]) 
{
	/* read command line */
	if ( itref_config(argc, argv) ) {
		return 1;
	}

	/* set up data structures */
	if ( itref_setup(n, bm, bn, s, &A, x, &rhs, &work) ) {
		return 2;
	}

	/* select solver */
	int (*cgimpl) (int bm, int bn, int n, void *A, int s, void *b, void *x, int *offs, double tol, int steps, void *work, unsigned long works, int lookahead, int async, double prof, int warmup, int cglog, int release, double orth_fac);  
	cgimpl = NULL;
	int cgidx = ompssapp_readid(ompssapp_ITREF, "solver", OMPSSAPP_CGPROF);
	switch ( cgidx ) {
		case OMPSSAPP_CG:
			cgimpl = &ompss_scg;
			break;
		case OMPSSAPP_CGMOD1:
			cgimpl = &ompss_scgmod1;
			break;
		case OMPSSAPP_CGPROF:
			cgimpl = &ompss_scgprof;
			break;
		default:
			break;
	}

	int n2 = n * n;
	int ns = n * s;
	float *SA = malloc(sizeof(float) * n2);
	float *srhs = malloc(sizeof(float) * ns);
	struct timeval start;

	float *sdb = malloc(2 * ns * 4 * sizeof(float));
	float *sd[] = { &sdb[0], &sdb[4*ns] };
	double *rb = malloc(2 * ns * sizeof(double));
	double *r[] = { &rb[0], &rb[ns] };
	float *srb = malloc(2 * ns * sizeof(float));
	float *sr[] = { &srb[0], &srb[ns] };
	double *r2normb = calloc(2*s, sizeof(double));
	double *r2norm[] = { &r2normb[0], &r2normb[s] };
	//float *sz = malloc(sizeof(float) * ns);

	async_t *sync = malloc(2*sizeof(async_t));
	async_setup(sync);
	async_init(sync, 2, n, bm, -1, 1);
	async_log(&sync[0], 256, "itref");

	/* convert A and rhs to single precision */
 	bblas_d2s(bm, bm, n, n, A, SA);
	bblas_d2s(bm, bn, n, s, rhs, srhs);

	unsigned long elapsed=0;
	int offs = 0;
	int k;
	for ( k=0; k<rep; ++k ) {
		memset(sd[1], 0, ns * 4 * sizeof(float));
		memset(sd[0], 0, ns * 4 * sizeof(float));
		//omp_set_num_threads(p);
		
		/* first estimate in single precision */
		cgimpl(bm, bn, n, SA, s, srhs, sd[1], &offs, prec, it, work, works, lookahead, async, profile, warmup, 0, release, orth_fac);

		/* residual in double precision */
		bblas_s2d(bm, bn, n, s, sd[1]+offs*n, x[1]); 
		bblas_dcopy(1, bm, bn, n, s, rhs, r[1]);
		bblas_dgemm(1, OMPSSBLAS_NTRANSP, OMPSSBLAS_NTRANSP, bm, bn, bm, n, s, n, -1.0, A, n, x[1], n, 1.0, r[1], n); 	

		/* iterative refinement */
		int broken = 0;  
		int brokec = 0;
		int brokeseq = 0;
		int prevb = 1;
		*(r2norm[1]) = DBL_MAX; 

		int i;
		for ( i=0; i<refit; ++i ) {
			printf("ITREF i: %d\n", i);
			int b = i & 0x1;
			gettimeofday(&start, NULL);

			bblas_d2s(bm, bn, n, s, r[prevb], sr[b]); 
			bblas_sset(bm, bn, n, 4 * s, 0.0, sd[b]);

			if ( broken ) {
				brokeseq = 0;
				broken = offs = 0; 
				//scg(bm, bn, n, SA, s, sr, sd, &offs, NULL, prec, it, work, works, lookahead, 0, scale);
			} else {
				/* A * s = r */
				int cgit = cgimpl(bm, bn, n, SA, s, sr[b], sd[b], &offs, prec, it, work, works, lookahead, async, profile, warmup, 0, release, orth_fac);
#if 0
				int broke = cgit < 4;
				broken = broke || (brokeseq > 2);
				int diverge = cgit < 16;
				brokeseq += diverge ? 1: 0;
				brokeseq = diverge ? brokeseq : 0;
				//brokec += diverg? 1: 0;
				//printf("broke %i brokeseq %i\n", broke, brokeseq);
#endif
				broken = 0;
			}

			#pragma omp taskwait on (*r2norm[prevb])

			double sr2norm = sqrt(*(r2norm[prevb]));
			//printf("IREF: %8i %i res %.16e\n", i-1, prevb, sr2norm); 
			struct timeval stop;
			gettimeofday(&stop, NULL);
			unsigned long itlaps = (stop.tv_sec - start.tv_sec) * 1e6 + stop.tv_usec - start.tv_usec;
			log_record(&sync[0], i-1, EVENT_RESIDUAL, itlaps, sr2norm);
//			printf("ITREF_RES: %.16e\n", sr2norm);

			/* Handle case of too small correction due to small Lanczos basis  
			if (snrm2_(&N, sz, &inc) < (float)cg_tol) {
				printf("INCREASE CG STEPS AT ITERATION: %d\n", I);
				    cg_steps = cg_steps + floor((double)(cg_steps)/2);
			}
			*/

			/* x (dp) = x (dp) + s (sp) */
			bblas_sdcpaxpy(bm, bn, n, s, 1.0, sd[b] + offs * n, x[prevb], x[b]); 

			/* r = b - A * x */
			bblas_dcopy(1, bm, bn, n, s, rhs, r[b]);
			bblas_dgemm(1, OMPSSBLAS_NTRANSP, OMPSSBLAS_NTRANSP, bm, bn, bm, n, s, n, -1.0, A, n, x[b], n, 1.0, r[b], n); 	

			bblas_ddot(i, &sync[1], bm, bn, n, s, r[b], r[b], r2norm[b]);

			prevb = b;
		}

		#pragma omp taskwait
	}

	free(SA);
	free(srhs);
	free(sdb);
	free(rb);
	free(srb);
	//free(sz);
	free(r2normb);

	async_fini(sync, 2);
	itref_cleanup(n, s, A, x, rhs, work);

	return 0;
}
