#include "itref_setup.h"

#include "fptype.h"
#include "fpblas.h"
#include "genmat.h"
#include "async_struct.h"
#include "convertmat.h"
#include "matfprint.h"
#include "matfread.h"
#include "ompss_apps.h"
#include "genmat_config.h"

#include <stdlib.h>
#include <stdio.h>

#if USE_SPARSE
#include "hb.h"
#include "hbconvrt.h"
#endif


extern unsigned long works;
extern char *rhsfname;
extern char *aname;
static float lmarker[4];


typedef struct {
	async_t sync[3]; 
	fp_t bstart;
} worklay_t;


int itref_setup(int n, int bm, int bn, int s, void **A, double *x[], double **rhs, float **work) 
{
#if USE_SPARSE
	hbmat_t *lA = *A;;
	*A = hb2hbh_symcsr(lA, bm, NULL, 0);
#else
	double *lA;
	float *fA;
	if ( aname != NULL ) {
		printf("read A\n");
		lA = *A = (double*) calloc(n*n, sizeof(double));
		fA = calloc(n*n, sizeof(float));
		FILE *laf;
		if ( laf = fopen(aname, "r")) {
			READ_MM2DENSE(laf, n*n, 1, fA);
			fclose(laf);
			int i;
			for (i=0; i<n*n; ++i)
				lA[i] = (double) fA[i];
		} else {
			fprintf(stderr, "setup: could not open %s\n", aname);
			return 7;
		}
	} else {
		printf("gen A\n");
		lA = *A = dgenmat_config(ompssapp_ITREF, "A", n, n, n, GENMATCONF_COVAR);
		int i;
		fA = calloc(n*n, sizeof(float));
		for (i=0; i<n*n; ++i)
			fA[i] = (float) lA[i];
		FPRINT_DENSE2MM("A.dat", "A", n*n, 1, fA, n*n);
	}
	if ( lA == NULL ) {
		fprintf(stderr, "setup: could not allocate A\n");
		return 5;
	}
#endif

	int ns = n * s;
	int dupl = 4;

	double *lx = malloc(2 * dupl * ns * sizeof(double));
	if ( lx == NULL ) {
		fprintf(stderr, "err: could not allocate x\n");
		return 1;
	}
	x[0] = &lx[0];
	x[1] = &lx[dupl*ns];

/* Construct or read the rhs (B) */
	double *lrhs;
	float *frhs;
	if ( rhsfname != NULL ) {
		printf("read rhs\n");
		lrhs = *rhs = (double*) calloc(ns, sizeof(double));
		frhs = calloc(ns, sizeof(float));
		if ( lrhs == NULL ) {
			fprintf(stderr, "err: could not allocate rhs\n");
			return 3;
		}

		FILE *rhsf;
		if ( rhsf = fopen(rhsfname, "r") ) {
			READ_MM2DENSE(rhsf, n*s, 1, frhs);
			fclose(rhsf);
			int i;
			for(i=0; i< n*s; ++i)
				lrhs[i] = (double) frhs[i];
		} else {
			fprintf(stderr, "setup: could not open %s\n", rhsfname);
			return 6;
		}
	} else {
		printf("gen rhs\n");
#if 0
		fp_t *lxstar = *xstar = GENMAT(n, s, n);
		if ( lxstar == NULL ) {
			fprintf(stderr, "err: could not allocate xstar\n");
			return 2;
		}
		FPRINT_DENSE2MM("xstar.mm", "xstar", n, s, lxstar, n);

		BLAS_gemm(OMPSSBLAS_TRANSP, OMPSSBLAS_NTRANSP, n, s, n, FP_ONE, lA, n, lxstar, n, FP_NOUGHT, lrhs, n); 
		// or rather mkl_dcsrsymv ("U", &n, mat.vval, mat.vptr, mat.vpos, x, rhs);
#endif
		lrhs = *rhs = dgenmat_config(ompssapp_ITREF, "rhs", n, s, n, GENMATCONF_COSTAS);
		frhs = calloc(n*s, sizeof(float));
		int i;
		for ( i = 0; i < n*s; ++i )
			frhs[i] = (float) lrhs[i];
		FPRINT_DENSE2MM("RHS.dat", "RHS", n*s, 1, frhs, n*s);
	}

/* align work buffer on 128-byte boundary */
	int nsalgn = ((ns + 0x7f ) >> 7 ) << 7;
	int salgn = ((s + 0x7f ) >> 7 ) << 7;
	int tmps = nsalgn * dupl * 4 + 3 * salgn * dupl;

	works = sizeof(worklay_t) + (tmps - 1) * sizeof(fp_t);  
	/* an additional 4 for the sentinels */
	if ( posix_memalign(work, 128, works + 4 * sizeof(float)) ) {
		fprintf(stderr, "setup: failed to allocate aligned memory\n");
		return 4;
	}

	worklay_t *worklay = (worklay_t*) *work;
	async_setup(worklay->sync);

	fp_t *marker = &worklay->bstart + tmps;
	marker[0] = lmarker[0] = FP_RAND();
	marker[1] = lmarker[1] = FP_RAND();
	marker[2] = lmarker[2] = FP_RAND();
	marker[3] = lmarker[3] = FP_RAND();

	works = tmps;


	return 0;
}


int itref_cleanup(int n, int s, void *A, double *x[], double *rhs, float *work) {
	worklay_t *worklay = (worklay_t*) work;

	async_t *sync = worklay->sync;
	async_fini(sync, 3);

	fp_t *marker = &worklay->bstart + works;
	int ok = marker[0] == lmarker[0] && marker[1] == lmarker[1] && marker[2] == lmarker[2] && marker[3] == lmarker[3];
	if ( ! ok ) {
		fprintf(stderr, "err: marker does not look good\n");
	}

	free(A);
	free(x[0]); 
	free(rhs);
	free(work);
}
